﻿''**************************************************************
'' MaxiCode Encoder for IDAutomation Barcode Fonts 2021 DEMO
'' © Copyright, 2021 IDAutomation.com, Inc. All rights reserved.
'' This function returns a text string as an Expression. This
'' string will generate the barcode when combined with the
'' correct barcode font.
'' Font to use: IDAutomation Maxicode
''**************************************************************
'' Limitations of the Demo Version:
'' The demo version of this product includes a fully
'' functional 2D font and is designed only for the
'' purpose of testing the font in various applications
'' and point sizes. To generate dynamic symbols,
'' a font encoder is required to format the data being
'' encoded into the 2D symbol.
'' Only static data can be provided for the encoders
'' in the demo version, because providing a working
'' encoder would reveal the complete source code,
'' which is only available in the licensed version.
'' 
'' Refer to the User Manual And Included Examples
'' https://www.idautomation.com/barcode-fonts/2d/maxicode/
''**************************************************************

Imports System
Imports System.Text

''' <summary>
''' Class constructor. 
''' </summary>
Public Class MaxiCode
#Region "Constants and tables"
    Friend Const HEX_SEPARATOR As Single = 0.1F
        Private Const PAD As Integer = 33
        Private Const ECI As Integer = 27
        Private Const NSHIFT As Integer = 31
        Private Const SHIFTA As Integer = 59
        Private Const SHIFTB As Integer = 59
        Private Const SHIFTC As Integer = 60
        Private Const LOCKC As Integer = 60
        Private Const SHIFTD As Integer = 61
        Private Const LOCKD As Integer = 61
        Private Const SHIFTE As Integer = 62
        Private Const LOCKE As Integer = 62
        Private Const LACHTB As Integer = 63
        Private Const LACHTA_B As Integer = 63 ' when in mode B
        Private Const LACHTA As Integer = 58
        Private Const PADC As Integer = 55
        Private Const PADE As Integer = 29
        Private Const SHIFT2A As Integer = 56
        Private Const SHIFT3A As Integer = 57
        Private Const GS As Char = Microsoft.VisualBasic.ChrW(29)
        Private Const RS As Char = Microsoft.VisualBasic.ChrW(30)

        ''' <summary>layout of the hexagons at 8 dpm
        ''' </summary>
        Friend pattern8dpm As Integer()() = {New Integer() {0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 1, 1, 0, 0}, New Integer() {0, 0, 1, 1, 1, 1, 0}, New Integer() {0, 1, 1, 1, 1, 1, 1}, New Integer() {0, 1, 1, 1, 1, 1, 1}, New Integer() {0, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 0}, New Integer() {0, 0, 0, 1, 1, 0, 0}}

        ''' <summary>layout of the hexagons at 12 dpm
        ''' </summary>
        Friend pattern12dpm As Integer()() = {New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 1, 1, 1, 0, 0, 0}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 0, 0}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 0}, New Integer() {0, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 0}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 0, 0}, New Integer() {0, 0, 0, 0, 1, 1, 1, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}}

        ''' <summary>layout of the hexagons at 16 dpm
        ''' </summary>
        Friend pattern16dpm As Integer()() = {New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}, New Integer() {0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0}}

        ''' <summary>layout of the hexagons at 20 dpm
        ''' </summary>
        Friend pattern20dpm As Integer()() = {New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0}, New Integer() {0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}, New Integer() {0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0}}

        ''' <summary>layout of the hexagons at 24 dpm
        ''' </summary>
        Friend pattern24dpm As Integer()() = {New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}, New Integer() {0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0}, New Integer() {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0}}
        Friend LookupTable As Object()() = {New Object() {0, "E", 0}, New Object() {1, "E", 1}, New Object() {2, "E", 2}, New Object() {3, "E", 3}, New Object() {4, "E", 4}, New Object() {5, "E", 5}, New Object() {6, "E", 6}, New Object() {7, "E", 7}, New Object() {8, "E", 8}, New Object() {9, "E", 9}, New Object() {10, "E", 10}, New Object() {11, "E", 11}, New Object() {12, "E", 12}, New Object() {13, "E", 13}, New Object() {13, "A", 0}, New Object() {14, "E", 14}, New Object() {15, "E", 15}, New Object() {16, "E", 16}, New Object() {17, "E", 17}, New Object() {18, "E", 18}, New Object() {19, "E", 19}, New Object() {20, "E", 20}, New Object() {21, "E", 21}, New Object() {22, "E", 22}, New Object() {23, "E", 23}, New Object() {24, "E", 24}, New Object() {25, "E", 25}, New Object() {26, "E", 26}, New Object() {27, "E", 30}, New Object() {28, "E", 32}, New Object() {28, "A", 28}, New Object() {28, "B", 28}, New Object() {28, "", 28}, New Object() {28, "D", 28}, New Object() {29, "", 29}, New Object() {29, "D", 29}, New Object() {29, "B", 29}, New Object() {29, "A", 29}, New Object() {29, "D", 33}, New Object() {30, "", 30}, New Object() {30, "D", 30}, New Object() {30, "B", 30}, New Object() {30, "E", 34}, New Object() {30, "A", 30}, New Object() {31, "E", 35}, New Object() {32, "D", 59}, New Object() {32, "A", 32}, New Object() {32, "E", 59}, New Object() {32, "", 59}, New Object() {32, "B", 47}, New Object() {33, "B", 53}, New Object() {34, "A", 34}, New Object() {35, "A", 35}, New Object() {36, "A", 36}, New Object() {37, "A", 37}, New Object() {38, "A", 38}, New Object() {39, "A", 39}, New Object() {40, "A", 40}, New Object() {41, "A", 41}, New Object() {42, "A", 42}, New Object() {43, "A", 43}, New Object() {44, "A", 44}, New Object() {44, "B", 48}, New Object() {45, "A", 45}, New Object() {46, "B", 49}, New Object() {46, "A", 46}, New Object() {47, "B", 50}, New Object() {47, "A", 47}, New Object() {48, "A", 48}, New Object() {49, "A", 49}, New Object() {50, "A", 50}, New Object() {51, "A", 51}, New Object() {52, "A", 52}, New Object() {53, "A", 53}, New Object() {54, "A", 54}, New Object() {55, "A", 55}, New Object() {56, "A", 56}, New Object() {57, "A", 57}, New Object() {58, "A", 58}, New Object() {58, "B", 51}, New Object() {59, "B", 37}, New Object() {60, "B", 38}, New Object() {61, "B", 39}, New Object() {62, "B", 40}, New Object() {63, "B", 41}, New Object() {64, "B", 52}, New Object() {65, "A", 1}, New Object() {66, "A", 2}, New Object() {67, "A", 3}, New Object() {68, "A", 4}, New Object() {69, "A", 5}, New Object() {70, "A", 6}, New Object() {71, "A", 7}, New Object() {72, "A", 8}, New Object() {73, "A", 9}, New Object() {74, "A", 10}, New Object() {75, "A", 11}, New Object() {76, "A", 12}, New Object() {77, "A", 13}, New Object() {78, "A", 14}, New Object() {79, "A", 15}, New Object() {80, "A", 16}, New Object() {81, "A", 17}, New Object() {82, "A", 18}, New Object() {83, "A", 19}, New Object() {84, "A", 20}, New Object() {85, "A", 21}, New Object() {86, "A", 22}, New Object() {87, "A", 23}, New Object() {88, "A", 24}, New Object() {89, "A", 25}, New Object() {90, "A", 26}, New Object() {91, "B", 42}, New Object() {92, "B", 43}, New Object() {93, "B", 44}, New Object() {94, "B", 45}, New Object() {95, "B", 46}, New Object() {96, "B", 0}, New Object() {97, "B", 1}, New Object() {98, "B", 2}, New Object() {99, "B", 3}, New Object() {100, "B", 4}, New Object() {101, "B", 5}, New Object() {102, "B", 6}, New Object() {103, "B", 7}, New Object() {104, "B", 8}, New Object() {105, "B", 9}, New Object() {106, "B", 10}, New Object() {107, "B", 11}, New Object() {108, "B", 12}, New Object() {109, "B", 13}, New Object() {110, "B", 14}, New Object() {111, "B", 15}, New Object() {112, "B", 16}, New Object() {113, "B", 17}, New Object() {114, "B", 18}, New Object() {115, "B", 19}, New Object() {116, "B", 20}, New Object() {117, "B", 21}, New Object() {118, "B", 22}, New Object() {119, "B", 23}, New Object() {120, "B", 24}, New Object() {121, "B", 25}, New Object() {122, "B", 26}, New Object() {123, "B", 32}, New Object() {124, "B", 54}, New Object() {125, "B", 34}, New Object() {126, "B", 35}, New Object() {127, "B", 36}, New Object() {128, "", 48}, New Object() {129, "", 49}, New Object() {130, "", 50}, New Object() {131, "", 51}, New Object() {132, "", 52}, New Object() {133, "", 53}, New Object() {134, "", 54}, New Object() {135, "", 55}, New Object() {136, "", 56}, New Object() {137, "", 57}, New Object() {138, "D", 47}, New Object() {139, "D", 48}, New Object() {140, "D", 49}, New Object() {141, "D", 50}, New Object() {142, "D", 51}, New Object() {143, "D", 52}, New Object() {144, "D", 53}, New Object() {145, "D", 54}, New Object() {146, "D", 55}, New Object() {147, "D", 56}, New Object() {148, "D", 57}, New Object() {149, "E", 48}, New Object() {150, "E", 49}, New Object() {151, "E", 50}, New Object() {152, "E", 51}, New Object() {153, "E", 52}, New Object() {154, "E", 53}, New Object() {155, "E", 54}, New Object() {156, "E", 55}, New Object() {157, "E", 56}, New Object() {158, "E", 57}, New Object() {159, "E", 36}, New Object() {160, "E", 37}, New Object() {161, "D", 37}, New Object() {162, "E", 38}, New Object() {163, "E", 39}, New Object() {164, "E", 40}, New Object() {165, "E", 41}, New Object() {166, "D", 42}, New Object() {167, "E", 43}, New Object() {168, "D", 38}, New Object() {169, "E", 44}, New Object() {170, "", 37}, New Object() {171, "D", 39}, New Object() {172, "", 38}, New Object() {173, "E", 45}, New Object() {174, "E", 46}, New Object() {175, "D", 40}, New Object() {176, "D", 41}, New Object() {177, "", 39}, New Object() {178, "", 40}, New Object() {179, "", 41}, New Object() {180, "D", 42}, New Object() {181, "", 42}, New Object() {182, "E", 47}, New Object() {183, "D", 43}, New Object() {184, "D", 44}, New Object() {185, "", 43}, New Object() {186, "", 44}, New Object() {187, "D", 45}, New Object() {188, "", 45}, New Object() {189, "", 46}, New Object() {190, "", 47}, New Object() {191, "D", 46}, New Object() {192, "", 0}, New Object() {193, "", 1}, New Object() {194, "", 2}, New Object() {195, "", 3}, New Object() {196, "", 4}, New Object() {197, "", 5}, New Object() {198, "", 6}, New Object() {199, "", 7}, New Object() {200, "", 8}, New Object() {201, "", 9}, New Object() {202, "", 10}, New Object() {203, "", 11}, New Object() {204, "", 12}, New Object() {205, "", 13}, New Object() {206, "", 14}, New Object() {207, "", 15}, New Object() {208, "", 16}, New Object() {209, "", 17}, New Object() {210, "", 18}, New Object() {211, "", 19}, New Object() {212, "", 20}, New Object() {213, "", 21}, New Object() {214, "", 22}, New Object() {215, "", 23}, New Object() {216, "", 24}, New Object() {217, "", 25}, New Object() {218, "", 26}, New Object() {219, "", 32}, New Object() {220, "", 33}, New Object() {221, "", 34}, New Object() {222, "", 35}, New Object() {223, "", 36}, New Object() {224, "D", 0}, New Object() {225, "D", 1}, New Object() {226, "D", 2}, New Object() {227, "D", 3}, New Object() {228, "D", 4}, New Object() {229, "D", 5}, New Object() {230, "D", 6}, New Object() {231, "D", 7}, New Object() {232, "D", 8}, New Object() {233, "D", 9}, New Object() {234, "D", 10}, New Object() {235, "D", 11}, New Object() {236, "D", 12}, New Object() {237, "D", 13}, New Object() {238, "D", 14}, New Object() {239, "D", 15}, New Object() {240, "D", 16}, New Object() {241, "D", 17}, New Object() {242, "D", 18}, New Object() {243, "D", 19}, New Object() {244, "D", 20}, New Object() {245, "D", 21}, New Object() {246, "D", 22}, New Object() {247, "D", 23}, New Object() {248, "D", 24}, New Object() {249, "D", 25}, New Object() {250, "D", 26}, New Object() {251, "D", 32}, New Object() {252, "", 33}, New Object() {253, "D", 34}, New Object() {254, "D", 35}, New Object() {255, "D", 36}}
#End Region


    ''' <summary>
    ''' Function, which creates a MaxiCode barcode, depending on the various input options chosen. 
    ''' </summary>
    ''' <param name="DataToEncode">The data that is encoded in the barcode.</param>
    ''' <param name="ApplyTilde">Enables processing of tilde and parentheses characters for purposes of encoding ASCII functions and GS1 application identifiers.</param>
    ''' <param name="EncMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
    ''' <param name="QuietZone">Define the size of a blank zone around the code.</param>
    ''' <param name="DotPM">Define the size of each module.</param>
    ''' <returns>A byte stream representation of vector Maxicode's raster image.</returns>
    Public Function EncodeMaxiCode(ByVal DataToEncode As String, ByVal Optional ApplyTilde As Boolean = True, ByVal Optional EncMode As EncodingModes = EncodingModes.Mode2, ByVal Optional QuietZone As Integer = 2, ByVal Optional DotPM As Integer = 16) As Byte()
            encodingMode = EncMode
        _encode(DataToEncode, ApplyTilde)
        Dim Result As Byte() = Nothing
            Dim k As Integer
        Dim intPad As Integer = 0
        Dim pixels As Integer = 0
        Dim bit As Integer = 0
        Dim bWidth As Integer = 0
        Dim rows As Integer = 33
        Dim cols As Integer = 30
        Dim ModuleSizeX As Integer = 0
        Dim ModuleSizeY As Integer = 0
        Dim SizeX As Integer = 19
        Dim SizeY As Integer = 17
        Dim Ray As Double = 16.0R * DotPM
        Dim dots()() As Integer = pattern24dpm

        If DotPM = 20 Then
                dots = pattern20dpm
                SizeX = 16
                SizeY = 14
                Ray = 14.0R * DotPM
            ElseIf DotPM = 16 Then
                dots = pattern16dpm
                SizeX = 14
                SizeY = 12
                Ray = 13.0R * DotPM
            ElseIf DotPM = 12 Then
                dots = pattern12dpm
                SizeX = 11
                SizeY = 9
                Ray = 11.0R * DotPM
            ElseIf DotPM = 8 Then
                dots = pattern8dpm
                SizeX = 7
                SizeY = 6
                Ray = 7.0R * DotPM
            End If

            ModuleSizeY = dots.Length
            ModuleSizeX = dots(0).Length
            If QuietZone < 0 Then QuietZone = 0
        Dim OffsetX As Integer = SizeX \ 2
        '			int OffsetY = ModuleSize * 1.5d;

        Dim bitmap = New Integer((rows + 1 + 2 * QuietZone) * SizeY - 1, (cols + 1 + 2 * QuietZone) * SizeX - 1) {}

            For y = 0 To rows - 1

                For x = 0 To cols - 1

                    If (y And 1) <> 0 Then
                        OffsetX = SizeX / 2
                    Else
                        OffsetX = 0
                    End If

                    If Matrix(y, x) = 1 Then
                        For n = 0 To ModuleSizeY - 1

                            For m = 0 To ModuleSizeX - 1

                                If dots(n)(m) = 1 Then
                                    bitmap(SizeY * (y + QuietZone) + n, SizeX * (x + QuietZone) + m + OffsetX) = 1
                                End If
                            Next
                        Next
                    End If
                Next
            Next

        Dim Cx As Double = (14 + QuietZone ) * SizeX + SizeX / 2
        Dim Cy As Double = (16 + QuietZone) * SizeY + SizeY / 2
        Dim cat As Double = 0

        For y = CInt(Cy - 6 * SizeY) To CInt(Cy + 6 * SizeY - 1)

            For x = CInt(Cx - 6 * SizeX) To CInt(Cx + 6 * SizeX - 1)
                cat = Math.Pow(x - Cx, 2) + Math.Pow(y - Cy, 2)

                If cat > Ray * 12 And cat <= Ray * 18 Or cat > Ray * 4 And cat <= Ray * 8 Or cat > 0.4 * Ray And cat <= Ray * 1.5 Then
                    bitmap(y, x) = 1
                End If
            Next
        Next

        cols = cols + 2 * QuietZone
            rows = rows + 2 * QuietZone
        bWidth = Math.Floor((cols * SizeX) / 8) + If(cols * SizeX Mod 8 > 0, 1, 0)
        Dim intTemp = bWidth Mod 4

            If intTemp <> 0 Then
                intPad = 4 - intTemp
            End If

            Result = New Byte(64 + (rows + 1) * SizeY * (bWidth + intPad) - 1) {}
            Result(0) = &H42
            Result(1) = &H4D

            For k = 0 To 8 - 1
                Result(2 + k) = 0
            Next

            Result(10) = &H3E

            For k = 0 To 3 - 1
                Result(11 + k) = 0
            Next

            Result(14) = &H28

            For k = 0 To 3 - 1
                Result(15 + k) = 0
            Next

            ' width
            Result(18) = CByte(cols * SizeX Mod 256)
            Result(19) = CByte(Math.Floor(cols * SizeX / 256))
            Result(20) = 0
            Result(21) = 0

            ' height
            Result(22) = CByte((rows + 1) * SizeY Mod 256)
            Result(23) = CByte(Math.Floor((rows + 1) * SizeY / 256))
            Result(24) = 0
            Result(25) = 0
            Result(26) = 1
            Result(27) = 0
            Result(28) = 1
            Result(29) = 0

            For k = 0 To 8 - 1
                Result(30 + k) = 0
            Next

            Result(38) = &H25
            Result(39) = &HE
            Result(40) = 0
            Result(41) = 0
            Result(42) = &H25
            Result(43) = &HE
            Result(44) = 0
            Result(45) = 0

            For k = 0 To 8 - 1
                Result(46 + k) = 0
            Next

            For k = 0 To 3 - 1
                Result(54 + k) = &HFF
            Next

            For k = 0 To 5 - 1
                Result(57 + k) = 0
            Next

            Dim pos = 62

            For y = (rows + 1) * SizeY - 1 To 0 Step -1
                pixels = 0
                bit = 8

                For x = 0 To cols * SizeX - 1
                    bit = bit - 1
                    If bitmap(y, x) = 1 Then pixels = pixels + CInt(Math.Pow(2, bit))

                    If bit = 0 Then
                        Result(pos) = CByte(pixels)
                        pos += 1
                        bit = 8
                        pixels = 0
                    End If
                Next

                If bit <> 8 Then
                    Result(pos) = CByte(pixels)
                    pos += 1
                End If

                pos = pos + intPad
            Next

            Return Result
        End Function


    ''' <summary>
    ''' Function, which creates a Maxicode barcode, depending on the various input options chosen. 
    ''' </summary>
    ''' <param name="DataToEncode">The data that is encoded in the barcode.</param>
    ''' <param name="ApplyTilde">Enables processing of tilde and parentheses characters for purposes of encoding ASCII functions and GS1 application identifiers.</param>
    ''' <param name="EncMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
    ''' <param name="VectorType">Define the type of vector image for output.</param>
    ''' <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
    ''' <param name="QuietZone">Define the size of a blank zone around the code.</param>
    ''' <param name="DotPM">Define the size of each module.</param>
    ''' <returns>A string representation of vector Maxicode's image.</returns>
    Public Function EncodeMaxiCode(ByVal DataToEncode As String, ByVal Optional ApplyTilde As Boolean = True, ByVal Optional EncMode As EncodingModes = EncodingModes.Mode2, ByVal Optional VectorType As VectorTypes = VectorTypes.VecEPS, ByVal Optional NewLine As String = vbCrLf, ByVal Optional QuietZone As Integer = 2, ByVal Optional DotPM As Integer = 16) As String
        encodingMode = EncMode
        If (NewLine = "") Then NewLine = vbCrLf
        _newline = NewLine
        _encode(DataToEncode, ApplyTilde) ', QuietZone);
        Dim rows = 33
        Dim cols = 30
        Dim strOut As StringBuilder = New StringBuilder("")
        Dim tempW = ""
        Dim tempH = ""
        Dim ModuleSize As Double = 4

        If DotPM = 20 Then
            ModuleSize = 3.5
        ElseIf DotPM = 16 Then
            ModuleSize = 3
        ElseIf DotPM = 12 Then
            ModuleSize = 2.1
        ElseIf DotPM = 8 Then
            ModuleSize = 1.5R
        End If

        If QuietZone < 0 Then QuietZone = 0
        Dim ModuleSizeX = ModuleSize * 4.9
        Dim ModuleSizeY = ModuleSize * 4.1
        Dim Ray = 0.4R * ModuleSizeX
        Dim OffsetX = ModuleSizeX * 0.5R
        Dim OffsetY = ModuleSizeY * 0.5R
        tempW = ((cols + 0.5 + 2 * QuietZone) * ModuleSizeX + OffsetX).ToString().Replace(",", ".")
        tempH = ((rows + 2 * QuietZone) * ModuleSizeY + OffsetY).ToString().Replace(",", ".")

        If VectorType = VectorTypes.VecSVG Then
            OffsetY = 0.5R * ModuleSizeY
            strOut.Append("<?xml version='1.0' standalone='no'?>" & _newline)
            strOut.Append("<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>" & _newline)
            strOut.Append("<svg width='" & tempW & "' height='" & tempH & "' version='1.1' xmlns='http://www.w3.org/2000/svg'>" & _newline)
            strOut.Append("  <g id='maxicode'>" & _newline)
            strOut.Append("    <g id='background' fill='#ffffff' stroke='#ffffff'>" & _newline)
            strOut.Append("      <rect x='0' y='0' width='" & tempW & "' height='" & tempH & "' />" & _newline)
            strOut.Append("    </g>" & _newline)
            strOut.Append("    <g id='pattern' fill='#000000' stroke='#000000'>" & _newline)

            For y = 0 To rows - 1

                For x = 0 To cols - 1

                    If (y And 1) <> 0 Then
                        OffsetX = ModuleSizeX
                    Else
                        OffsetX = ModuleSizeX * 0.5
                    End If

                    If Matrix(y, x) = 1 Then
                        strOut.Append("    <g id='R" & y & "" & x & "' >" & _newline)
                        strOut.Append("      <polygon points='" & (((x + QuietZone) * ModuleSizeX + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & "," & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") & " " & (((x + QuietZone) * ModuleSizeX + OffsetX).ToString()).Replace(",", ".") & "," & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray).ToString()).Replace(",", ".") & " " & (((x + QuietZone) * ModuleSizeX - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & "," & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") & " " & (((x + QuietZone) * ModuleSizeX - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & "," & (((y + QuietZone) * ModuleSizeY + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") & " " & (((x + QuietZone) * ModuleSizeX + OffsetX).ToString()).Replace(",", ".") & "," & (((y + QuietZone) * ModuleSizeY + OffsetY + Ray).ToString()).Replace(",", ".") & " " & (((x + QuietZone) * ModuleSizeX + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & "," & (((y + QuietZone) * ModuleSizeY + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") & " " & "' />" & NewLine)
                        strOut.Append("    </g>" & _newline)
                    End If
                Next
            Next

            strOut.Append("    </g>" & _newline)
            Dim cx = 14 + QuietZone
            Dim cy = 16 + QuietZone
            OffsetX = ModuleSizeX * 0.5R
            Ray = 0.58R * ModuleSizeX
            strOut.Append("    <g id='target'>" & _newline)
            strOut.Append(("      <circle style='fill:#000000; stroke:none;' cx='" & (cx * ModuleSizeX + OffsetX).ToString() & "' cy='" & (cy * ModuleSizeY + OffsetY).ToString() & "' r='" & (Ray * 7.59).ToString() & "' />").Replace(",", ".") & NewLine)
            strOut.Append(("      <circle style='fill:#ffffff; stroke:none;' cx='" & (cx * ModuleSizeX + OffsetX).ToString() & "' cy='" & (cy * ModuleSizeY + OffsetY).ToString() & "' r='" & (Ray * 6.27).ToString() & "' />").Replace(",", ".") & NewLine)
            strOut.Append(("      <circle style='fill:#000000; stroke:none;' cx='" & (cx * ModuleSizeX + OffsetX).ToString() & "' cy='" & (cy * ModuleSizeY + OffsetY).ToString() & "' r='" & (Ray * 4.96).ToString() & "' />").Replace(",", ".") & NewLine)
            strOut.Append(("      <circle style='fill:#ffffff; stroke:none;' cx='" & (cx * ModuleSizeX + OffsetX).ToString() & "' cy='" & (cy * ModuleSizeY + OffsetY).ToString() & "' r='" & (Ray * 3.64).ToString() & "' />").Replace(",", ".") & NewLine)
            strOut.Append(("      <circle style='fill:#000000; stroke:none;' cx='" & (cx * ModuleSizeX + OffsetX).ToString() & "' cy='" & (cy * ModuleSizeY + OffsetY).ToString() & "' r='" & (Ray * 2.31).ToString() & "' />").Replace(",", ".") & NewLine)
            strOut.Append(("      <circle style='fill:#ffffff; stroke:none;' cx='" & (cx * ModuleSizeX + OffsetX).ToString() & "' cy='" & (cy * ModuleSizeY + OffsetY).ToString() & "' r='" & Ray.ToString() & "' />").Replace(",", ".") & NewLine)
            strOut.Append("    </g>" & _newline)
            strOut.Append("  </g>" & _newline)
            strOut.Append("</svg>" & _newline)
        ElseIf VectorType = VectorTypes.VecEPS Then
            strOut.Append("%!PS-Adobe-3.0 EPSF-3.0" & _newline)
            strOut.Append("%%Creator: IDAutomation.com" & _newline)
            strOut.Append("%%Title: maxicode" & _newline)
            strOut.Append("%%CreationDate: " & Date.Now.ToString("ddd mmm d hh:mm:ss yyyy") & _newline)
            strOut.Append("%%BoundingBox: 0 0 " & tempW & " " & tempH & _newline)
            strOut.Append("%%DocumentData: Clean7Bit" & _newline)
            strOut.Append("%%EndComments" & _newline)
            strOut.Append("%%BeginProlog" & _newline)
            strOut.Append("/bd { bind def } bind def" & _newline)
            strOut.Append("/incompound false def" & _newline)
            strOut.Append("/m { moveto } bd" & _newline)
            strOut.Append("/l { lineto } bd" & _newline)
            strOut.Append("/*U { /incompound false def f} bd" & _newline)
            strOut.Append("/k { setcmykcolor } bd" & _newline)
            strOut.Append("/c { curveto } bd" & _newline)
            strOut.Append("/F { incompound not {fill} if } bd" & _newline)
            strOut.Append("/f { closepath F } bd" & _newline)
            strOut.Append("/S { stroke } bd" & _newline)
            strOut.Append("/*u { /incompound true def } bd" & _newline)
            strOut.Append("/K { k } bd" & _newline)
            strOut.Append("%%EndProlog" & _newline)
            strOut.Append("%%BeginSetup" & _newline)
            strOut.Append("%%EndSetup" & _newline)
            strOut.Append("0.000 0.000 0.000 0.000 k" & _newline)
            strOut.Append("0 0 m" & _newline)
            strOut.Append("0 0 l" & _newline)
            strOut.Append(tempW & " 0 l" & _newline)
            strOut.Append(tempW & " " & tempH & " l" & _newline)
            strOut.Append("0 " & tempH & " l" & _newline)
            strOut.Append("f" & _newline)
            strOut.Append("*U" & _newline)
            ModuleSizeX = ModuleSize * 4.9
            ModuleSizeY = ModuleSize * 4.1
            Ray = 0.5R * ModuleSizeX
            'int n = rows * ModuleSizeY;
            strOut.Append("0.000 0.000 0.000 1.000 k" & _newline)

            For y = 0 To rows - 1

                For x = 0 To cols - 1

                    If (y And 1) <> 0 Then
                        OffsetX = ModuleSizeX
                    Else
                        OffsetX = ModuleSizeX * 0.5
                    End If

                    If Matrix(rows - 1 - y, x) = 1 Then
                        strOut.Append((((x + QuietZone) * ModuleSizeX + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") & " m" & _newline)
                        strOut.Append((((x + QuietZone) * ModuleSizeX + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") & " l" & _newline)
                        strOut.Append((((x + QuietZone) * ModuleSizeX + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray).ToString()).Replace(",", ".") & " l" & _newline)
                        strOut.Append((((x + QuietZone) * ModuleSizeX - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") & " l" & _newline)
                        strOut.Append((((x + QuietZone) * ModuleSizeX - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") & " l" & _newline)
                        strOut.Append((((x + QuietZone) * ModuleSizeX + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY + Ray).ToString()).Replace(",", ".") & " l" & _newline)
                        strOut.Append((((x + QuietZone) * ModuleSizeX + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") & " " & (((y + QuietZone) * ModuleSizeY + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") & " l" & _newline)
                        strOut.Append("f" & _newline)
                    End If
                Next
            Next

            Dim sngL As Double
            OffsetX = ModuleSizeX * 0.5
            OffsetY = ModuleSizeX * 0.5
            Dim Cx = 14 + QuietZone
            Dim Cy = 16 + QuietZone
            Ray = 0.57R * ModuleSizeX * 7.59
            sngL = 4.0R / 3.0R * (Math.Sqrt(2.0R) - 1.0R) * Ray
            strOut.Append("0.000 0.000 0.000 1.000 k" & _newline)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " m").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append("f" & NewLine)
            Ray = 0.57R * ModuleSizeX * 6.27
            sngL = 4.0R / 3.0R * (Math.Sqrt(2.0R) - 1.0R) * Ray
            strOut.Append("0.000 0.000 0.000 0.000 k" & _newline)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " m").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append("f" & NewLine)
            Ray = 0.57R * ModuleSizeX * 4.96
            sngL = 4.0R / 3.0R * (Math.Sqrt(2.0R) - 1.0R) * Ray
            strOut.Append("0.000 0.000 0.000 1.000 k" & _newline)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " m").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append("f" & NewLine)
            Ray = 0.57R * ModuleSizeX * 3.64
            sngL = 4.0R / 3.0R * (Math.Sqrt(2.0R) - 1.0R) * Ray
            strOut.Append("0.000 0.000 0.000 0.000 k" & _newline)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " m").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append("f" & NewLine)
            Ray = 0.57R * ModuleSizeX * 2.31
            sngL = 4.0R / 3.0R * (Math.Sqrt(2.0R) - 1.0R) * Ray
            strOut.Append("0.000 0.000 0.000 1.000 k" & _newline)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " m").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append("f" & NewLine)
            Ray = 0.57R * ModuleSizeX
            sngL = 4.0R / 3.0R * (Math.Sqrt(2.0R) - 1.0R) * Ray
            strOut.Append("0.000 0.000 0.000 0.000 k" & _newline)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " m").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY - Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY - sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX - sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() & " " & (Cy * ModuleSizeY + OffsetY + Ray).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY + sngL).ToString() & " " & (Cx * ModuleSizeX + OffsetX + Ray).ToString() & " " & (Cy * ModuleSizeY + OffsetY).ToString() & " c").Replace(",", ".") & NewLine)
            strOut.Append("f" & NewLine)
            strOut.Append("*U" & _newline)
            strOut.Append("%%Trailer" & _newline)
            strOut.Append("%%EOF" & _newline)
        ElseIf VectorType = VectorTypes.VecHTML Then
            strOut.Append("<meta http-equiv='X-UA-Compatible' content='IE=10' />")
            strOut.Append("<div style='font-size:0;background-color:#ffffff;position:relative;width:" & tempW & "px;height:" & tempH & "px;'>" & _newline)

            For y = 0 To rows - 1

                For x = 0 To cols - 1

                    If (y And 1) <> 0 Then
                        OffsetX = ModuleSizeX
                    Else
                        OffsetX = ModuleSizeX * 0.5
                    End If

                    If Matrix(y, x) = 1 Then
                        strOut.Append("<div style='font-size:" & (1.3R * ModuleSizeX).ToString().Replace(",", ".") & "px;color:#000000;position:absolute;left:" & ((x + QuietZone) * ModuleSizeX + OffsetX).ToString().Replace(",", ".") & "px;top:" & ((y + QuietZone) * ModuleSizeY + OffsetY).ToString().Replace(",", ".") & "px;'>&#x2B22;</div>" & _newline)
                    End If
                Next
            Next

            Dim cx = 14 + QuietZone
            Dim cy = 16 + QuietZone
            OffsetX = ModuleSizeX * 1.0R
            OffsetY = ModuleSizeY * 1.5R
            Ray = 0.58R * ModuleSizeX
            strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" & "background:#000000;" & "width:" & (2 * Ray * 7.59).ToString().Replace(",", ".") & "px;" & "height:" & (2 * Ray * 7.59).ToString().Replace(",", ".") & "px;" & "position:absolute;left:" & (cx * ModuleSizeX + OffsetX - Ray * 7.59).ToString().Replace(",", ".") & "px;" & "top:" & (cy * ModuleSizeY + OffsetY - Ray * 7.59).ToString().Replace(",", ".") & "px;'>&nbsp</div>" & NewLine)
            strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" & "background:#ffffff;" & "width:" & (2 * Ray * 6.27).ToString().Replace(",", ".") & "px;" & "height:" & (2 * Ray * 6.27).ToString().Replace(",", ".") & "px;" & "position:absolute;left:" & (cx * ModuleSizeX + OffsetX - Ray * 6.27).ToString().Replace(",", ".") & "px;" & "top:" & (cy * ModuleSizeY + OffsetY - Ray * 6.27).ToString().Replace(",", ".") & "px;'>&nbsp</div>" & NewLine)
            strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" & "background:#000000;" & "width:" & (2 * Ray * 4.96).ToString().Replace(",", ".") & "px;" & "height:" & (2 * Ray * 4.96).ToString().Replace(",", ".") & "px;" & "position:absolute;left:" & (cx * ModuleSizeX + OffsetX - Ray * 4.96).ToString().Replace(",", ".") & "px;" & "top:" & (cy * ModuleSizeY + OffsetY - Ray * 4.96).ToString().Replace(",", ".") & "px;'>&nbsp</div>" & NewLine)
            strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" & "background:#ffffff;" & "width:" & (2 * Ray * 3.64).ToString().Replace(",", ".") & "px;" & "height:" & (2 * Ray * 3.64).ToString().Replace(",", ".") & "px;" & "position:absolute;left:" & (cx * ModuleSizeX + OffsetX - Ray * 3.64).ToString().Replace(",", ".") & "px;" & "top:" & (cy * ModuleSizeY + OffsetY - Ray * 3.64).ToString().Replace(",", ".") & "px;'>&nbsp</div>" & NewLine)
            strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" & "background:#000000;" & "width:" & (2 * Ray * 2.31).ToString().Replace(",", ".") & "px;" & "height:" & (2 * Ray * 2.31).ToString().Replace(",", ".") & "px;" & "position:absolute;left:" & (cx * ModuleSizeX + OffsetX - Ray * 2.31).ToString().Replace(",", ".") & "px;" & "top:" & (cy * ModuleSizeY + OffsetY - Ray * 2.31).ToString().Replace(",", ".") & "px;'>&nbsp</div>" & NewLine)
            strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" & "background:#ffffff;" & "width:" & (2 * Ray).ToString().Replace(",", ".") & "px;" & "height:" & (2 * Ray).ToString().Replace(",", ".") & "px;" & "position:absolute;left:" & (cx * ModuleSizeX + OffsetX - Ray).ToString().Replace(",", ".") & "px;" & "top:" & (cy * ModuleSizeY + OffsetY - Ray).ToString().Replace(",", ".") & "px;'>&nbsp</div>" & NewLine)
            strOut.Append("</div>" & _newline)
        End If

        Return strOut.ToString()
    End Function



    ''' <summary>
    ''' Function, which creates a MaxiCode barcode, depending on the various input options chosen. 
    ''' </summary>
    ''' <param name="DataToEncode">The data that is encoded in the barcode.</param>
    ''' <param name="ApplyTilde">Enables processing of tilde and parentheses characters for purposes of encoding ASCII functions and GS1 application identifiers.</param>
    ''' <param name="EncMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
    ''' <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
    ''' <returns>Encoded text for use with the 2D barcode font.</returns>
    Public Function EncodeMaxiCode(ByVal DataToEncode As String, ByVal Optional ApplyTilde As Boolean = True, ByVal Optional EncMode As EncodingModes = EncodingModes.Mode2, ByVal Optional NewLine As String = vbCrLf) As String
        encodingMode = EncMode


        Matrix = {{1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1},
                    {0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0},
                    {1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
                    {1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
                    {1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1},
                    {0, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0},
                    {1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1},
                    {0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0},
                    {1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0},
                    {1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0},
                    {1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
                    {1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, -2, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0},
                    {1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1, 0},
                    {0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0},
                    {0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0},
                    {1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 0},
                    {1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0},
                    {1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0},
                    {0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0},
                    {1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
                    {0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
                    {0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, -3, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0},
                    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1},
                    {1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0},
                    {0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0},
                    {1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1},
                    {0, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0},
                    {0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1},
                    {1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0},
                    {0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1},
                    {1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0},
                    {1, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0}}



        If (NewLine = "") Then NewLine = vbCrLf

        _newline = NewLine
        _encode(DataToEncode, ApplyTilde)

        Dim StringToPrint As StringBuilder = New StringBuilder()

        Try
            Dim row = 0

            While row < 33

                For col = 0 To 30 - 1

                    If col = 14 AndAlso row = 12 Then
                        StringToPrint.Append("V")
                    ElseIf col = 14 AndAlso row = 18 Then
                        StringToPrint.Append("X")
                    ElseIf Matrix(row, col) = 0 Then

                        If Matrix(row + 1, col) = 0 Then
                            If Matrix(row + 2, col) = 0 Then
                                StringToPrint.Append("0")
                            Else
                                StringToPrint.Append("1")
                            End If
                        Else

                            If Matrix(row + 2, col) = 0 Then
                                StringToPrint.Append("2")
                            Else
                                StringToPrint.Append("3")
                            End If
                        End If
                    Else

                        If Matrix(row + 1, col) = 0 Then
                            If Matrix(row + 2, col) = 0 Then
                                StringToPrint.Append("4")
                            Else
                                StringToPrint.Append("5")
                            End If
                        Else

                            If Matrix(row + 2, col) = 0 Then
                                StringToPrint.Append("6")
                            Else
                                StringToPrint.Append("7")
                            End If
                        End If
                    End If
                Next

                StringToPrint.Append(_newline)

                If row + 4 < 33 Then
                    For col = 0 To 30 - 1

                        If col = 14 AndAlso row + 3 = 15 Then
                            StringToPrint.Append("W")
                        ElseIf Matrix(row + 3, col) = 0 Then

                            If Matrix(row + 4, col) = 0 Then
                                If Matrix(row + 5, col) = 0 Then
                                    StringToPrint.Append("0")
                                ElseIf Matrix(row + 5, col) = -2 Then
                                    StringToPrint.Append("E")
                                Else
                                    StringToPrint.Append("N")
                                End If
                            Else

                                If Matrix(row + 5, col) = 0 Then
                                    StringToPrint.Append("O")
                                ElseIf Matrix(row + 5, col) = -2 Then
                                    StringToPrint.Append("F")
                                Else
                                    StringToPrint.Append("P")
                                End If
                            End If
                        ElseIf Matrix(row + 3, col) = -3 Then

                            If Matrix(row + 4, col) = 0 Then
                                If Matrix(row + 5, col) = 0 Then
                                    StringToPrint.Append("p")
                                Else
                                    StringToPrint.Append("q")
                                End If
                            Else

                                If Matrix(row + 5, col) = 0 Then
                                    StringToPrint.Append("r")
                                Else
                                    StringToPrint.Append("s")
                                End If
                            End If
                        Else

                            If Matrix(row + 4, col) = 0 Then
                                If Matrix(row + 5, col) = 0 Then
                                    StringToPrint.Append("Q")
                                ElseIf Matrix(row + 5, col) = -2 Then
                                    StringToPrint.Append("G")
                                Else
                                    StringToPrint.Append("R")
                                End If
                            Else

                                If Matrix(row + 5, col) = 0 Then
                                    StringToPrint.Append("S")
                                ElseIf Matrix(row + 5, col) = -2 Then
                                    StringToPrint.Append("H")
                                Else
                                    StringToPrint.Append("T")
                                End If
                            End If
                        End If
                    Next

                    StringToPrint.Append(_newline)
                End If

                row = row + 6
            End While

        Catch localException As Exception
            Console.Out.WriteLine("Error in EncodeMaxiCode " & localException.ToString())
        End Try

        Return StringToPrint.ToString()
    End Function


#Region "Internal and Private variables"

    ''' <summary>
    ''' This is a standard value for newline
    ''' </summary>

    Private _newline As String = ""

        ''' <summary>
        ''' This is a matrix values that we will draw the code on
        ''' </summary>
        Friend Matrix As Integer(,) = Nothing

        ''' <summary>
        ''' This is a layout mask that we will draw the hexagons on
        ''' </summary>
        Friend layout As Integer()() = Nothing
        ''' <summary>
        ''' The data to be encoded
        ''' </summary>
        Friend UserData As String = "IDAutomation.com, Inc."

        ''' <summary>
        ''' The type of encoding that should be done.  Based on the Maxicode standards
        ''' </summary>
        Friend encodingMode As EncodingModes = EncodingModes.Mode2

        ''' <summary>
        ''' A 3 digit number representing the Ship To Country Code
        ''' </summary>
        Private countryCode As String = ""

        ''' <summary>
        ''' A 3 digit number representing the Class of Service
        ''' </summary>
        Private classOfService As String = ""

        ''' <summary>
        ''' A 5 or 9 digit number representing the zip code
        ''' </summary>
        Friend zipCode As String = ""

        ''' <summary>
        ''' Used for Structured Append:  Number of Symbol in structured appended mode
        ''' </summary>
        Friend numberOfCodes As Integer = 1

        ''' <summary>
        ''' Used for Structured Append:  Position of this symbol in the sequence of the structured appended
        ''' </summary>
        Friend positionOfCode As Integer = 1
#End Region



#Region "Mehtods"
    ''' <summary>
    ''' This is the driver for encoding Maxicode data into a format for barcoding
    ''' </summary>
    Friend Sub _encode(ByVal DataToBarcode As String, ByVal Optional ApplyTilde As Boolean = True)

        Matrix = {{1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1},
                    {0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0},
                    {1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
                    {1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
                    {1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1},
                    {0, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0},
                    {1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1},
                    {0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0},
                    {1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0},
                    {1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0},
                    {1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
                    {1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, -2, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0},
                    {1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1, 0},
                    {0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0},
                    {0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0},
                    {1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 0},
                    {1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0},
                    {1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0},
                    {0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0},
                    {1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
                    {0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
                    {0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, -3, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0},
                    {0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1},
                    {1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0},
                    {0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
                    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0},
                    {1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1},
                    {0, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0},
                    {0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1},
                    {1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0},
                    {0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1},
                    {1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0},
                    {1, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0}}


        Return
        End Sub 'End _encode



End Class


'ENUMERATIONS:

''' <summary>
''' The mode of compression and encoding used in the symbol.
''' </summary>
Public Enum EncodingModes
    ''' <summary>
    ''' Modes 2 is designed for use in the transport industry.
    ''' </summary>
    Mode2
    ''' <summary>
    ''' Modes 3 is designed for use in the transport industry.
    ''' </summary>
    Mode3
    ''' <summary>
    ''' Mode 4 encodes up to 93 characters or 138 digits.
    ''' </summary>
    Mode4
    ''' <summary>
    ''' Mode 5 encodes up to 77 characters and provides more error correction capabilities than mode 4.
    ''' </summary>
    Mode5
    ''' <summary>
    ''' Mode 6 indicates that the symbol encodes a message used to program a reader system (scanner).
    ''' </summary>
    Mode6
End Enum

''' <summary>
''' Various types of vector image output.
''' </summary>
Public Enum VectorTypes

    ''' <summary>
    ''' Returns a string with the SVG rapresentation. 
    ''' </summary>
    VecSVG

    ''' <summary>
    ''' Return a string with the EPS rapresentation.
    ''' </summary>
    VecEPS

    ''' <summary>
    ''' Return a string with the HTML rapresentation.
    ''' </summary>
    VecHTML
End Enum
#End Region
