﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using IDAutomation.NetStandard.MaxiCode.FontEncoder;

namespace BarcodeGenerationExample
{
    public partial class TabTest : Form
    {
        public TabTest()
        {
            InitializeComponent();
            cmbMode.SelectedIndex = 1;
        }


        private void TestTab()
        {
            MaxiCode MX = new MaxiCode();
            string encodedText = "";
            string dataToEncode = "";
            bool applyTilde = true; // ' use ~ for special encoding
            var encodingMode = EncodingModes.Mode2;
            var raster = default(byte[]);
            var vector = default(string);
            var testData = new string[] { "[)>~03001~02996336091062~029840~029002~0291Z14647438~029UPSN~029410E1W~029195~029~0291/1~029~029Y~029135Reo~029TAMPA~029FL~030~004",
                                          "[)>~03001~029961Z00004951~029UPSN~02906X610~029159~0291234567~0291/1~029~029Y~029634 ALPHA DR~029PITTSBURGH~029PA~030~004",
                                          "[)>~03001~02996336091062~029840~029002~0299Z94647438~029UPSN~029410E1W~029195~029~0291/1~029~029Y~029550 North Cypress Point Way~029TAMPA~029FL~030~004",
                                          "[)>~03001~02996336297224~029840~029003~0291Z06393640~029UPSN~029750Y1E~03007VI3%S%QCMH:7HQVE'483/5~028EL..EF+&O(0CYAY~029" + '"' + "%H(J~010~030~004",
                                          "[)>~03001~029961Z00136111~029UPSN~029123X56~029057~029~029001/001~029011~029N~029~029 WEST SWINDON~029~030~004",
                                          "MODE 4 MAX DATA TEST. THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG. THE QUICK BROWN FOX JUMPS.",
                                          "Mode 4 Max Data test. The quick brown fox jumps over the lazy dog. The quick brown fox",
                                          "MODE 5 MAX DATA TEST. THE QUICK BROWN FOX JUMPS OVER THE LAZY DOG. THE QUICK",
                                          "Mode 5 Max Data test. The quick brown fox jumps over the lazy dog.",
                                          "[)>~03001~02996336091062~029840~029002~0291Z9999999999999999~029UPSN~029410E1W~029195~029~0291/1~029~029Y~0295500 CYPRESS PT DR~029BRADENTO~029FL~030~004",
                                          "[)>~03001~02996336091062~029840~029002~0291Z9999999999999999~029UPSN~029410E1W~029195~029~0291/1~029~029Y~0295500 CYPRESS PT DR~029BRADENTON~029FL~030~004"
                                        };
            var testMode = new int[] { 2,
                                                 2,
                                                 2,
                                                 2,
                                                 3,
                                                 4,
                                                 4,
                                                 5,
                                                 5,
                                                 2,
                                                 2
                                        };

            int i;
            for (i = 0; i < testData.Length; i++)
            {
                dataToEncode = testData[i];
                switch (testMode[i])
                {
                    case 2:
                        encodingMode = EncodingModes.Mode2;
                        break;
                    case 3:
                        encodingMode = EncodingModes.Mode3;
                        break;
                    case 4:
                        encodingMode = EncodingModes.Mode4;
                        break;
                    case 5:
                        encodingMode = EncodingModes.Mode5;
                        break;
                    default:
                        encodingMode = EncodingModes.Mode2;
                        break;
                }
                if (cmbMode.SelectedIndex == 2) // ' BMP Then raster image output
                {
                    raster = (byte[])MX.EncodeMaxiCode(dataToEncode, applyTilde, encodingMode, 1, 16);
                }
                else if (cmbMode.SelectedIndex == 3)
                {
                    vector = MX.EncodeMaxiCode(dataToEncode, applyTilde, encodingMode, VectorTypes.VecSVG, "", 1, 16);
                }
                else if (cmbMode.SelectedIndex == 4)
                {
                    vector = MX.EncodeMaxiCode(dataToEncode, applyTilde, encodingMode, VectorTypes.VecHTML, "", 1, 16);
                }
                else if (cmbMode.SelectedIndex < 2)
                {
                    encodedText = MX.EncodeMaxiCode(dataToEncode, applyTilde, encodingMode, null);
                }

                renderBarcodeToTab("MaxiCode" + (i + 1).ToString(), dataToEncode, encodingMode.ToString(), encodedText, raster, vector);
                if (chkSaveEPS.Checked)
                {
                    vector = MX.EncodeMaxiCode(dataToEncode, applyTilde, encodingMode, VectorTypes.VecEPS, "", 1, 16);
                    File.WriteAllText("MaxiCode" + (i + 1).ToString() + ".EPS", vector);
                }
            }


        }

        private void renderBarcodeToTab(string tabName, string dataToEncode, string encodingMode, string encodedText, byte[] raster, string vector)
        {
            int LL = 20;
            int LTOP = 10;
            int YSpacing = 10;
            var tp = new TabPage(tabName);
            tp.Font = new Font("Consolas", 10.0f, FontStyle.Regular, GraphicsUnit.Point, 0);
            tp.AutoScroll = true; // 'allow scrolling to fit output
            TabControl1.TabPages.Add(tp); // 'add tab to page

            // ' TOP - LEFT
            var showDTEText = new Label();
            showDTEText.Name = "showDTEText";
            showDTEText.Location = new Point(LL, LTOP);
            showDTEText.AutoSize = true;
            showDTEText.Text = "DATA ENCODED: ";
            showDTEText.Font = new Font("Microsoft Sans Serif", 10.0f, FontStyle.Bold, GraphicsUnit.Point, 0);
            tp.Controls.Add(showDTEText);
            var showDTE = new Label();
            showDTE.Name = "showDTE";
            showDTE.Location = new Point(LL, showDTEText.Location.Y + showDTEText.Height + YSpacing);
            showDTE.AutoSize = true;
            showDTE.Text = formatText(dataToEncode);
            tp.Controls.Add(showDTE);


            // ' encodingMode
            var lblEncMode = new Label();
            lblEncMode.Location = new Point(LL, showDTE.Location.Y + showDTE.Height + YSpacing);
            lblEncMode.Name = "lblECM";
            lblEncMode.Size = new Size(200, 13);
            lblEncMode.Font = new Font("Microsoft Sans Serif", 8.25f, FontStyle.Bold, GraphicsUnit.Point, 0);
            lblEncMode.Text = "ENC MODE: " + encodingMode.ToUpper();
            tp.Controls.Add(lblEncMode);

            // 'barcode
            var lblBarcodeTitle = new Label();
            lblBarcodeTitle.AutoSize = true;
            lblBarcodeTitle.Location = new Point(LL, lblEncMode.Location.Y + lblEncMode.Height + YSpacing);
            lblBarcodeTitle.Name = "lblDTETitle";
            // 'lblBarcodeTitle.Size = New System.Drawing.Size(83, 13)
            lblBarcodeTitle.Text = "BARCODE:";
            lblBarcodeTitle.Font = new Font("Microsoft Sans Serif", 8.25f, FontStyle.Bold, GraphicsUnit.Point, 0);
            tp.Controls.Add(lblBarcodeTitle);

            var lblBarcode = new Label();
            lblBarcode.AutoSize = true;
            lblBarcode.Location = new Point(LL, lblBarcodeTitle.Location.Y + lblBarcodeTitle.Height + YSpacing);
            lblBarcode.Name = "lblBarcodeBox";
            lblBarcode.Text = encodedText;
            if (cmbMode.SelectedIndex == 0)
            {
                lblBarcode.Font = new Font("Courier New", 10.0f, FontStyle.Regular, GraphicsUnit.Point, 0);
                tp.Controls.Add(lblBarcode);
            }
            else if (cmbMode.SelectedIndex == 1)
            {
                lblBarcode.Font = new Font("IDAutomation MaxiCode", 14.5f, FontStyle.Regular, GraphicsUnit.Point, 0);
                tp.Controls.Add(lblBarcode);

                // 'encoded data
                var lblEncDataTitle = new Label();
                lblEncDataTitle.AutoSize = true;
                lblEncDataTitle.Location = new Point(LL, lblBarcode.Location.Y + lblBarcode.Height + YSpacing);
                lblEncDataTitle.Name = "lblDTETitle";
                // 'lblEncDataTitle.Size = New System.Drawing.Size(83, 13)
                lblEncDataTitle.Text = "ENCODED RESULT:";
                lblEncDataTitle.Font = new Font("Microsoft Sans Serif", 8.25f, FontStyle.Bold, GraphicsUnit.Point, 0);
                tp.Controls.Add(lblEncDataTitle);
                var lblEncData = new Label();
                lblEncData.Location = new Point(LL, lblEncDataTitle.Location.Y + lblEncDataTitle.Height + YSpacing);
                lblEncData.Name = "lblEncData";
                lblEncData.AutoSize = true;
                lblEncData.Text = encodedText;
                tp.Controls.Add(lblEncData);
            }
            else if (cmbMode.SelectedIndex == 2)
            {
                var picImage = new PictureBox();
                picImage.AutoSize = true;
                picImage.Location = new Point(LL, lblBarcodeTitle.Location.Y + lblBarcodeTitle.Height + YSpacing);
                picImage.Name = "picCode";
                var mStream = new MemoryStream(raster);
                var img1 = Image.FromStream(mStream);
                picImage.Image = img1;
                tp.Controls.Add(picImage);
            }
            else if (cmbMode.SelectedIndex == 3 | cmbMode.SelectedIndex == 4)
            {
                var WebBrowser = new WebBrowser();
                WebBrowser.Location = new Point(LL, lblBarcodeTitle.Location.Y + lblBarcodeTitle.Height + YSpacing);
                WebBrowser.Width = tp.Width - WebBrowser.Left;
                WebBrowser.Height = tp.Height - WebBrowser.Top;
                WebBrowser.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Bottom;
                WebBrowser.DocumentText = vector;
                tp.Controls.Add(WebBrowser);
            }
        }

        private string formatText(string inText)
        {
            string formatTextRet = default;
            int maxWidth = 175; // ' width Of text shown In window
            string formattedString = "";
            for (int i = 0, loopTo = inText.Length - 1; maxWidth >= 0 ? i <= loopTo : i >= loopTo; i += maxWidth)
                formattedString += SafeSubstring(inText, i, maxWidth) + Environment.NewLine;
            formatTextRet = formattedString;
            return formatTextRet;
        }

        private string SafeSubstring(string value, int startIndex, int length)
        {
            string SafeSubstringRet = default;
            if (!string.IsNullOrEmpty(value))
            {
                if (value.Length - startIndex < length)
                    length = value.Length - startIndex;
                value = value.Substring(startIndex, length);
            }

            SafeSubstringRet = value;
            return SafeSubstringRet;
        }

        private void cmbMode_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            TabControl1.TabPages.Clear();
            TestTab();
        }

        private void chkSaveEPS_CheckedChanged_1(object sender, EventArgs e)
        {
            TabControl1.TabPages.Clear();
            TestTab();
        }
    }
}
